#!/usr/bin/env python3
"""  remap.py

A wrapper to run REMAP from within python.

Copyright 2022, Uli Wortmann, uli.wortmann@utoronto.ca

"""
from __future__ import annotations
import numpy as np
import pandas as pd


def get_delta(r: float, conc: np.array, a: int, b: int) -> float:
    """Calculate delta values from total concentration and concentration of the
    light isotope

    r = isotope standart reference ratio
    conc = nd.arrays with concentration values
    a & b = index values

    returns the delta value d
    """
    d = (
        (
            (conc[a, :] - conc[b, :]) / (conc[b, :] + (conc[b, :] == 0))
            - (r * (conc[b, :] != 0))
        )
        / r
        * 1000
    )
    return d

def run_remap(infile: pathlib.Path)-> pd.DataFrame:
    """ This function starts the octace process and rusn the
    start_remap.m script.

    infile:  A fully qualified file name of pathlib object

    The function returns:
    c: np.array of constants
    conc: np.array of concentration values
    r: ???
    v: ???
    par: np.array  of parameters
    """
    from oct2py import octave
    import pandas as pd
    import site

    # set the path where the remap_lib and start remap files live
    ip = site.getsitepackages()[0]
    octave.addpath(f"{ip}/remappy/")
    
    # call the remap solver
    [c, conc, r, v, par] = octave.start_remap(str(infile), nout=5)

    # transfer results into dataframe. Use the tolist() method to convert
    # from numpy data type
    df = pd.DataFrame()

    df["Depth"] = c.depth[0, :].tolist()
    df["Substrate"] = conc[0, :].tolist()
    df["Product"] = conc[2, :].tolist()
    df["Precipitate"] = conc[4, :].tolist()
    df["f"] = par.f[0, :].tolist()

    if c.ref != 0:
        df["Substrate d"] = get_delta(c.ref, conc, 0, 1)
        df["Product d"] = get_delta(c.ref, conc, 2, 3)
        df["Precipitate d"] = get_delta(c.ref, conc, 4, 5)
        # set all zeros to NaN, except in first row
        cols = ["Substrate d", "Product d", "Precipitate d"]
        df[cols] = df[cols][1:].replace({"0": np.nan, 0: np.nan})
    else:
        df["Substrate d"] = np.zeros(int(c.imax)) * np.nan
        df["Product d"] = np.zeros(int(c.imax)) * np.nan
        df["Precipitate d"] = np.zeros(int(c.imax)) * np.nan

    return df

if __name__ == "__main__":
    import numpy as np
    import pathlib as pl
    import sys
    import argparse
    import matplotlib.pyplot as plt
    import pandas as pd
    import plot_it

    # --------------------------- setup cmd line parsing with argparse ------------------ #
    parser = argparse.ArgumentParser(
        description="""REMAP version 0.11, Copyright (C) Boris Chernyavsky
        If you publish results which involve REMAP
        Please cite  Chernyavsky & Wortmann (2007)
        doi:10.1029/2006GC001442"""
    )
    parser.add_argument(
        "config",
        help="model configuration file, i.e. foo.rmp",
        type=str,
    )
    parser.add_argument(
        "-ns",
        "--no-save",
        help="do not save results, defaults to False",
        dest="save",
        action="store_false",
    )
    parser.add_argument(
        "-nd",
        "--no-display",
        help="do not save results, defaults to False",
        dest="display",
        action="store_false",
    )
    parser.add_argument(
        "-md",
        "--measured-data",
        help="""provide a csv file with measured data.
        This file needs to have the same format as the remap results file""",
        dest="measured_data",
        action="store",
        metavar="myfile.csv",
    )
    parser.add_argument(
        "-f",
        "--figure-name",
        dest="fig_name",
        help="figure name - defaults to 'default_fig.pdf'",
        default="default_fig.pdf",
        metavar="my_fig.pdf",
        type=str,
    )
    parser.add_argument(
        "-c",
        "--columns",
        help="""provide list of column indices to restrict which columns are
                 plotted. Note that that the first columns has an index of 0.""",
        dest="columns",
        action="store",
        metavar="0 1 4",
        default=None,
        nargs="+",
        type=int,
    )

    parser.set_defaults(feature=True)  # set defaults
    args = parser.parse_args()  # get cmd line arguments
    fn: str = args.config  # getinput file
    cwd: pl.Path = pl.Path.cwd()  # get the current working directory
    infile: pl.Path = pl.Path(f"{cwd}/{fn}")  # fully qualified file name
    outfile = infile.with_suffix(".csv")  # set the results file name

    df = run_remap(infile)  # run remap

    # --------------------------- save results as csv fie ------------------------------------ #
    if args.save:
        df.to_csv(outfile, index=False)

    # --------------------------- plot data ------------- ------------------------------------ #
    if args.display:

        if args.measured_data:
            dfn: str = args.measured_data  # getinput file
            cwd: pl.Path = pl.Path.cwd()  # get the current working directory
            datafile: pl.Path = pl.Path(f"{cwd}/{dfn}")  # fully qualified file name

            if not datafile.exists():  # check if file exist
                raise FileNotFoundError(f"Cannot find file {datafile}")

            md: pd.DataFrame = pd.read_csv(datafile)  # read data

            ax, fig = plot_it.plot_data([df, md], columns=args.columns)

        else:
            ax, fig = plot_it.plot_data(df, columns=args.columns)

        fig.tight_layout()
        plt.style.use("ggplot")
        fig.savefig(args.fig_name)
        plt.show



import pathlib as pl
import pandas as pd
import matplotlib.pyplot as plt
import numpy as np


def plot_data(plot_data, **kwargs) -> list:
    """plot_data = dataframe object or file name (or list). Both need
    to list the column header in row 1, and depth in column 1

    Optional arguments:

    fig = optional canvas handle, otherwise figure will be created
    ax = see fig
    columns = optional list, otherwise plot all columns of df
    xlabels = optional list of strings
    ylabel = optional, defaults to Depth [mbsf]

    returns the figure and graph handles

    """

    # provide defaults if none given
    defaults = {
        "fig": None,
        "ax": None,
        "columns": None,
        "xlabels": None,
        "ylabel": "Depth [mbsf]",
    }

    # check optional arguments
    dc = defaults.copy()
    for k, v in dc.items():
        if k in kwargs:
            defaults[k] = kwargs[k]

    # check for invalid keywords
    for k, v in kwargs.items():
        if k not in defaults:
            raise ValueError(f"{k} is an invalid keywords")

    # make sure we have a list
    if not isinstance(plot_data, list):
        plot_data = [plot_data]

    if defaults["fig"]:
        fig = defaults["fig"]
    else:
        fig = None

    xlabels = defaults["xlabels"]

    # loop over list elements
    j = 0
    for df in plot_data:
        # test what data we have
        if isinstance(df, str):
            cwd: pl.Path = pl.Path.cwd()  # get the current working directory
            fqfn: pl.Path = pl.Path(f"{cwd}/{df}")  # fully qualified file name

            if not fqfn.exists():  # check if file exist
                raise FileNotFoundError(f"Cannot find file {fqfn}")

            df = pd.read_csv(fqfn)

        elif not isinstance(df, pd.DataFrame):
            raise ValueError("df must be filename object or dataframe")

        # get number of columns in dataframe
        headers: list = list(df.columns)

        # check if we only print a selection
        if defaults["columns"]:
            cols = defaults["columns"]
        else:
            cols = range(len(headers) - 1)

        # set line color
        color = f"C{j}"
        # create figure object if necessary
        if not fig:
            fig, ax = plt.subplots(1, len(cols))
            fig.set_size_inches(3 * len(cols), 6)

        # loop over data (ignore depth column)
        depth = df.iloc[:, 0]
        i = 0
        u = 0
        for o in headers[1:]:
            if i in cols:
                if j == 0:
                    ax[u].plot(df[o], depth, color=color, label=o)
                else:
                    ax[u].scatter(df[o], depth, color=color, label=o)
                    
                if xlabels:
                    ax[u].set_xlabel(xlabels[i])
                else:
                    ax[u].set_xlabel(f"{o}")
                if j == 0:
                    ax[u].invert_yaxis()
                ax[u].legend()
                u = u + 1
            i = i + 1
        j = j + 1
    ax[0].set_ylabel(defaults["ylabel"])
    return [ax, fig]


# what do to when used as stand alone program
if __name__ == "__main__":
    import argparse
    import sys

    parser = argparse.ArgumentParser(
        description="""plot_it.py: plots remap stytle output data.
                                   I.e., the first column must contain depths data,
                                   and all other columns must contain data that
                                   varies with depth.

    The first row must contain headers. NaN and missing data may not be handled
    gracefully. Data must be in csv format
    """
    )
    parser.add_argument(
        "data_filename",
        help="one or more filenames with data in csv format",
        nargs="+",
        type=str,
    )
    parser.add_argument(
        "-f",
        "--figure-name",
        dest="fig_name",
        help="figure name - defaults to 'default_fig.pdf'",
        default="default_fig.pdf",
        metavar="my_fig.pdf",
        type=str,
    )
    parser.add_argument(
        "-c",
        "--columns",
        help="""provide list of column indices to restrict which columns are
             plotted. Note that that the first columns has an index of 0.""",
        dest="columns",
        action="store",
        metavar="[0, 1, 4]",
        default=None,
        nargs="+",
        type=int,
    )
    

    args = parser.parse_args()  # get cmd line arguments

    ax, fig = plot_data(args.data_filename, columns=args.columns)
    fig.tight_layout()
    plt.style.use("ggplot")
    plt.show
    fig.savefig(args.fig_name)
